@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 from __future__ import annotations
 
 import sys
+import threading
 from collections.abc import Callable, Iterable
 from copy import deepcopy
 from typing import (
@@ -78,6 +79,7 @@ from streamlit.elements.progress import ProgressMixin
 from streamlit.elements.pyplot import PyplotMixin
 from streamlit.elements.snow import SnowMixin
 from streamlit.elements.space import SpaceMixin
+from streamlit.elements.spinner import SpinnerMixin
 from streamlit.elements.text import TextMixin
 from streamlit.elements.toast import ToastMixin
 from streamlit.elements.vega_charts import VegaChartsMixin
@@ -99,8 +101,9 @@ from streamlit.elements.widgets.slider import SliderMixin
 from streamlit.elements.widgets.text_widgets import TextWidgetsMixin
 from streamlit.elements.widgets.time_widgets import TimeWidgetsMixin
 from streamlit.elements.write import WriteMixin
-from streamlit.errors import StreamlitAPIException
-from streamlit.proto import Block_pb2, ForwardMsg_pb2
+from streamlit.errors import NoSessionContext, StreamlitAPIException
+from streamlit.proto import Block_pb2
+from streamlit.proto.ForwardMsg_pb2 import ForwardMsg
 from streamlit.proto.RootContainer_pb2 import RootContainer
 from streamlit.runtime import caching
 from streamlit.runtime.scriptrunner import enqueue_message as _enqueue_message
@@ -114,6 +117,7 @@ if TYPE_CHECKING:
     from streamlit.cursor import Cursor
     from streamlit.elements.lib.built_in_chart_utils import AddRowsMetadata
     from streamlit.elements.lib.layout_utils import LayoutConfig
+    from streamlit.proto.Element_pb2 import Element as ElementProto
 
 MAX_DELTA_BYTES: Final[int] = 14 * 1024 * 1024  # 14MB
 
@@ -122,6 +126,7 @@ Value = TypeVar("Value")
 # Type aliases for Ancestor Block Types
 BlockType: TypeAlias = str
 AncestorBlockTypes: TypeAlias = Iterable[BlockType]
+ForwardMsgCreator: TypeAlias = Callable[[], ForwardMsg]
 
 
 _use_warning_has_been_displayed: bool = False
@@ -210,6 +215,7 @@ class DeltaGenerator(
     SliderMixin,
     SnowMixin,
     SpaceMixin,
+    SpinnerMixin,
     JsonMixin,
     TextMixin,
     TextWidgetsMixin,
@@ -429,6 +435,13 @@ class DeltaGenerator(
     def id(self) -> str:
         return str(id(self))
 
+    def _get_transient_cursor(self) -> cursor.Cursor:
+        cursor = self._active_dg._cursor
+        if cursor is None:
+            raise NoSessionContext("Cursor is not set")
+
+        return cursor.get_transient_cursor()
+
     def _get_delta_path_str(self) -> str:
         """Returns the element's delta path as a string like "[0, 2, 3, 1]".
 
@@ -484,7 +497,7 @@ class DeltaGenerator(
         _maybe_print_fragment_callback_warning()
 
         # Copy the marshalled proto into the overall msg proto
-        msg = ForwardMsg_pb2.ForwardMsg()
+        msg = ForwardMsg()
         msg_el_proto = getattr(msg.delta.new_element, delta_type)
         msg_el_proto.CopyFrom(element_proto)
 
@@ -565,7 +578,7 @@ class DeltaGenerator(
         if dg._root_container is None or dg._cursor is None:
             return dg
 
-        msg = ForwardMsg_pb2.ForwardMsg()
+        msg = ForwardMsg()
         msg.metadata.delta_path[:] = dg._cursor.delta_path
         msg.delta.add_block.CopyFrom(block_proto)
 
@@ -608,6 +621,68 @@ class DeltaGenerator(
 
         return block_dg
 
+    def _transient(
+        self,
+        element_proto: ElementProto,
+        layout_config: LayoutConfig | None = None,
+    ) -> tuple[ForwardMsgCreator, ForwardMsgCreator]:
+        """Provides the factory functions for creating and clearing transient elements.
+        It preserves the delta path, transient index, and the set of transient elements.
+
+        Returns a tuple of two functions:
+        - create_transient_element: Creates the new transient element.
+        - clear_transient_element: Clears the transient element.
+        """
+        transient_cursor = self._get_transient_cursor()
+        delta_path = transient_cursor.delta_path
+        transient_index = transient_cursor.transient_index
+        transient_elements = transient_cursor.transient_elements
+
+        if layout_config:
+            if layout_config.height is not None:
+                element_proto.height_config.CopyFrom(
+                    get_height_config(layout_config.height)
+                )
+            if layout_config.width is not None:
+                element_proto.width_config.CopyFrom(
+                    get_width_config(layout_config.width)
+                )
+
+        # Revalidate the use of the lock here in the event
+        # better support threading/multiprocessing in Streamlit
+        transient_lock = threading.Lock()
+
+        def create_transient_element() -> ForwardMsg:
+            with transient_lock:
+                transient_elements[transient_index] = element_proto
+
+                create_msg = ForwardMsg()
+                create_msg.metadata.delta_path[:] = delta_path
+                create_msg.metadata.cacheable = False
+                # Make sure the transient message is set as it will
+                # not be set if there are no transient elements
+                create_msg.delta.new_transient.SetInParent()
+                for e in transient_elements:
+                    create_msg.delta.new_transient.elements.add().CopyFrom(e)
+
+            return create_msg
+
+        def clear_transient_element() -> ForwardMsg:
+            with transient_lock:
+                if transient_index in transient_elements:
+                    del transient_elements[transient_index]
+
+                clear_msg = ForwardMsg()
+                clear_msg.metadata.delta_path[:] = delta_path
+                clear_msg.metadata.cacheable = False
+                clear_msg.delta.new_transient.SetInParent()
+                for e in transient_elements:
+                    clear_msg.delta.new_transient.elements.add().CopyFrom(e)
+
+            return clear_msg
+
+        return create_transient_element, clear_transient_element
+
 
 def _writes_directly_to_sidebar(dg: DeltaGenerator) -> bool:
     in_sidebar = any(a._root_container == RootContainer.SIDEBAR for a in dg._ancestors)
