@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,6 +28,7 @@ from streamlit.runtime.scriptrunner_utils.script_run_context import (
     get_script_run_ctx,
 )
 from streamlit.runtime.state import register_widget
+from streamlit.string_util import validate_icon_or_emoji
 
 if TYPE_CHECKING:
     from types import TracebackType
@@ -86,6 +87,7 @@ class Dialog(DeltaGenerator):
         *,
         dismissible: bool = True,
         width: DialogWidth = "small",
+        icon: str | None = None,
         on_dismiss: Literal["ignore", "rerun"] | WidgetCallback = "ignore",
     ) -> Dialog:
         # Validation for on_dismiss parameter
@@ -99,26 +101,41 @@ class Dialog(DeltaGenerator):
         block_proto.dialog.title = title
         block_proto.dialog.dismissible = dismissible
         block_proto.dialog.width = _process_dialog_width_input(width)
+        block_proto.dialog.icon = validate_icon_or_emoji(icon)
+
+        # Compute a stable identity for the dialog based on its attributes.
+        # This ID is used in the frontend to distinguish between different dialogs
+        # and prevent showing stale content from a previous dialog (issue #10907).
+        # It's also used for widget registration when on_dismiss is activated
+        # but we don't want this to be a widget in its default case since it
+        # would change the behavior (especially with fragments).
+        element_id = compute_and_register_element_id(
+            "dialog",
+            user_key=None,
+            key_as_main_identity=False,
+            dg=parent,
+            title=title,
+            dismissible=dismissible,
+            width=width,
+            icon=icon,
+            on_dismiss=str(on_dismiss) if not callable(on_dismiss) else "callback",
+        )
+        # The block.id is used to identify the dialog in the frontend to
+        # prevent showing stale content from a previous dialog.
+        # For actual widget registration, we set the dialog.id also
+        # below. This will activate the widget behavior on dismiss.
+        block_proto.id = element_id
 
         # Handle on_dismiss functionality
         is_dismiss_activated = on_dismiss != "ignore"
-        element_id = None
 
         if is_dismiss_activated:
-            # Register as widget when on_dismiss is activated
-
+            # Register the dialog as a widget when on_dismiss is activated.
+            # The same element_id is used for widget registration.
             ctx = get_script_run_ctx()
 
-            element_id = compute_and_register_element_id(
-                "dialog",
-                user_key=None,
-                key_as_main_identity=False,
-                dg=parent,
-                title=title,
-                dismissible=dismissible,
-                width=width,
-                on_dismiss=str(on_dismiss) if not callable(on_dismiss) else "callback",
-            )
+            # Setting the dialog.id will activate the rerun on dismiss functionality
+            # in the frontend (we might add a dedicated flag later)
             block_proto.dialog.id = element_id
 
             register_widget(
