@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@ from typing import TYPE_CHECKING, Any, Final, TypeVar, overload
 from streamlit import config, logger
 from streamlit.dataframe_util import OptionSequence, convert_anything_to_list
 from streamlit.errors import StreamlitAPIException
+from streamlit.runtime.state import get_session_state
 from streamlit.runtime.state.common import RegisterWidgetResult
 from streamlit.type_util import (
     check_python_comparable,
@@ -282,3 +283,100 @@ def create_mappings(
         formatted_options,
         formatted_option_to_option_mapping,
     )
+
+
+def validate_and_sync_value_with_options(
+    current_value: T | None,
+    opt: Sequence[T],
+    default_index: int | None,
+    key: str | int | None,
+) -> tuple[T | None, bool]:
+    """Validate current value against options, resetting session state if invalid.
+
+    This function has a side-effect: if the value is not found in the options
+    and a key is provided, it will update session state with the new value.
+
+    Parameters
+    ----------
+    current_value
+        The current widget value to validate.
+    opt
+        The sequence of valid options.
+    default_index
+        The default index to reset to if value is invalid.
+    key
+        The widget key for session state updates.
+
+    Returns
+    -------
+    tuple[T | None, bool]
+        A tuple of (validated_value, value_was_reset).
+    """
+    if current_value is None:
+        return current_value, False
+
+    # Check if current value is still in the new options
+    try:
+        index_(opt, current_value)
+        return current_value, False
+    except ValueError:
+        # Value not in options - reset to default
+        if default_index is not None and len(opt) > 0:
+            new_value: T | None = opt[default_index]
+        else:
+            new_value = None
+
+        if key is not None:
+            # Update session_state so subsequent accesses in this run
+            # return the corrected value. Use reset_state_value to avoid
+            # the "cannot be modified after widget instantiated" error.
+            get_session_state().reset_state_value(str(key), new_value)
+
+        return new_value, True
+
+
+def validate_and_sync_multiselect_value_with_options(
+    current_values: list[T] | list[T | str],
+    opt: Sequence[T],
+    key: str | int | None,
+) -> tuple[list[T] | list[T | str], bool]:
+    """Validate multiselect values against options, syncing session state if needed.
+
+    This function has a side-effect: if any values are filtered out and a key
+    is provided, it will update session state with the filtered list.
+
+    Unlike selectbox which resets to a default when the value is invalid,
+    multiselect filters out invalid values and keeps the valid ones.
+
+    Parameters
+    ----------
+    current_values
+        The current list of selected values to validate.
+    opt
+        The sequence of valid options.
+    key
+        The widget key for session state updates.
+
+    Returns
+    -------
+    tuple[list[T] | list[T | str], bool]
+        A tuple of (validated_values, values_were_filtered).
+    """
+    if not current_values:
+        return current_values, False
+
+    valid_values: list[T | str] = []
+    for value in current_values:
+        try:
+            index_(opt, value)
+            valid_values.append(value)
+        except ValueError:  # noqa: PERF203
+            pass
+
+    if len(valid_values) == len(current_values):
+        return current_values, False
+
+    if key is not None:
+        get_session_state().reset_state_value(str(key), valid_values)
+
+    return valid_values, True
