@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@ from __future__ import annotations
 
 from dataclasses import dataclass
 from textwrap import dedent
-from typing import TYPE_CHECKING, Any, Literal, TypeAlias, cast
+from typing import TYPE_CHECKING, Any, Final, Literal, TypeAlias, cast
 
 from streamlit.dataframe_util import OptionSequence, convert_anything_to_list
 from streamlit.elements.lib.layout_utils import (
@@ -38,13 +38,48 @@ from streamlit.string_util import AnyNumber, clean_text, from_number
 
 if TYPE_CHECKING:
     from streamlit.delta_generator import DeltaGenerator
+    from streamlit.elements.lib.column_types import NumberFormat
 
 
 Value: TypeAlias = AnyNumber | str | None
 Delta: TypeAlias = AnyNumber | str | None
-DeltaColor: TypeAlias = Literal["normal", "inverse", "off"]
+DeltaColor: TypeAlias = Literal[
+    "normal",
+    "inverse",
+    "off",
+    "red",
+    "orange",
+    "yellow",
+    "green",
+    "blue",
+    "violet",
+    "gray",
+    "grey",
+    "primary",
+]
 DeltaArrow: TypeAlias = Literal["auto", "up", "down", "off"]
 
+# Mapping from delta_color string values to proto enum values
+_DELTA_COLOR_TO_PROTO: Final[dict[str, MetricProto.MetricColor.ValueType]] = {
+    "red": MetricProto.MetricColor.RED,
+    "orange": MetricProto.MetricColor.ORANGE,
+    "yellow": MetricProto.MetricColor.YELLOW,
+    "green": MetricProto.MetricColor.GREEN,
+    "blue": MetricProto.MetricColor.BLUE,
+    "violet": MetricProto.MetricColor.VIOLET,
+    "gray": MetricProto.MetricColor.GRAY,
+    "grey": MetricProto.MetricColor.GRAY,
+    "primary": MetricProto.MetricColor.PRIMARY,
+}
+
+# Valid delta_color values for validation
+_VALID_DELTA_COLORS: Final[set[str]] = {
+    "normal",
+    "inverse",
+    "off",
+    *_DELTA_COLOR_TO_PROTO.keys(),
+}
+
 
 @dataclass(frozen=True)
 class MetricColorAndDirection:
@@ -69,14 +104,10 @@ class MetricMixin:
         chart_data: OptionSequence[Any] | None = None,
         chart_type: Literal["line", "bar", "area"] = "line",
         delta_arrow: DeltaArrow = "auto",
+        format: str | NumberFormat | None = None,
     ) -> DeltaGenerator:
         r"""Display a metric in big bold font, with an optional indicator of how the metric changed.
 
-        Tip: If you want to display a large number, it may be a good idea to
-        shorten it using packages like `millify <https://github.com/azaitsev/millify>`_
-        or `numerize <https://github.com/davidsa03/numerize>`_. E.g. ``1234`` can be
-        displayed as ``1.2k`` using ``st.metric("Short number", millify(1234))``.
-
         Parameters
         ----------
         label : str
@@ -97,21 +128,40 @@ class MetricMixin:
             .. _st.markdown: https://docs.streamlit.io/develop/api-reference/text/st.markdown
 
         value : int, float, decimal.Decimal, str, or None
-             Value of the metric. ``None`` is rendered as a long dash.
+            Value of the metric. ``None`` is rendered as a long dash.
+
+            The value can optionally contain GitHub-flavored Markdown, subject
+            to the same limitations described in the ``label`` parameter.
 
         delta : int, float, decimal.Decimal, str, or None
-            Indicator of how the metric changed, rendered with an arrow below
-            the metric. If delta is negative (int/float) or starts with a minus
-            sign (str), the arrow points down and the text is red; else the
-            arrow points up and the text is green. If None (default), no delta
-            indicator is shown.
+            Amount or indicator of change in the metric. An arrow is shown next
+            to the delta, oriented according to its sign:
 
-        delta_color : "normal", "inverse", or "off"
-             If "normal" (default), the delta indicator is shown as described
-             above. If "inverse", it is red when positive and green when
-             negative. This is useful when a negative change is considered
-             good, e.g. if cost decreased. If "off", delta is  shown in gray
-             regardless of its value.
+            - If the delta is ``None`` or an empty string, no arrow is shown.
+            - If the delta is a negative number or starts with a minus sign,
+              the arrow points down and the delta is red.
+            - Otherwise, the arrow points up and the delta is green.
+
+            You can modify the display, color, and orientation of the arrow
+            using the ``delta_color`` and ``delta_arrow`` parameters.
+
+            The delta can optionally contain GitHub-flavored Markdown, subject
+            to the same limitations described in the ``label`` parameter.
+
+        delta_color : str
+            The color of the delta and chart. This can be one of the following:
+
+            - ``"normal"`` (default): The color is red when the delta is
+              negative and green otherwise.
+            - ``"inverse"``: The color is green when the delta is negative and
+              red otherwise. This is useful when a negative change is
+              considered good, like a decrease in cost.
+            - ``"off"``: The color is gray regardless of the delta.
+            - A named color from the basic palette: The chart and delta are the
+              specified color regardless of their value. This can be one of the
+              following: ``"red"``, ``"orange"``, ``"yellow"``, ``"green"``,
+              ``"blue"``, ``"violet"``, ``"gray"``/``"grey"``, or
+              ``"primary"``.
 
         help : str or None
             A tooltip that gets displayed next to the metric label. Streamlit
@@ -166,6 +216,9 @@ class MetricMixin:
             be used. Each value will be cast to ``float`` internally by
             default.
 
+            The chart uses the color of the delta indicator, which can be
+            modified using the ``delta_color`` parameter.
+
         chart_type : "line", "bar", or "area"
             The type of sparkline chart to display. This can be one of the
             following:
@@ -185,97 +238,122 @@ class MetricMixin:
             - ``"off"``: No arrow is shown, but the delta value remains
               visible.
 
+        format : str or None
+            A format string controlling how numbers are displayed for ``value``
+            and ``delta``. The format is only applied if the value or delta is
+            numeric. If the value or delta is a string with non-numeric
+            characters, the format is ignored. The format can be one of the
+            following values:
+
+            - ``None`` (default): No formatting is applied.
+            - ``"plain"``: Show the full number without any formatting (e.g. "1234.567").
+            - ``"localized"``: Show the number in the default locale format (e.g. "1,234.567").
+            - ``"percent"``: Show the number as a percentage (e.g. "123456.70%").
+            - ``"dollar"``: Show the number as a dollar amount (e.g. "$1,234.57").
+            - ``"euro"``: Show the number as a euro amount (e.g. "€1,234.57").
+            - ``"yen"``: Show the number as a yen amount (e.g. "¥1,235").
+            - ``"accounting"``: Show the number in an accounting format (e.g. "1,234.00").
+            - ``"bytes"``: Show the number in a byte format (e.g. "1.2KB").
+            - ``"compact"``: Show the number in a compact format (e.g. "1.2K").
+            - ``"scientific"``: Show the number in scientific notation (e.g. "1.235E3").
+            - ``"engineering"``: Show the number in engineering notation (e.g. "1.235E3").
+            - printf-style format string: Format the number with a printf
+              specifier, like ``"%d"`` to show a signed integer (e.g. "1234") or
+              ``"%.2f"`` to show a float with 2 decimal places.
+
         Examples
         --------
-        **Example 1: Show a metric**
+                **Example 1: Show a metric**
 
-        >>> import streamlit as st
-        >>>
-        >>> st.metric(label="Temperature", value="70 °F", delta="1.2 °F")
+                >>> import streamlit as st
+                >>>
+                >>> st.metric(label="Temperature", value="70 °F", delta="1.2 °F")
 
-        .. output::
-            https://doc-metric-example1.streamlit.app/
-            height: 210px
+                .. output::
+                    https://doc-metric-example1.streamlit.app/
+                    height: 210px
 
-        **Example 2: Create a row of metrics**
+                **Example 2: Create a row of metrics**
 
-        ``st.metric`` looks especially nice in combination with ``st.columns``.
+                ``st.metric`` looks especially nice in combination with ``st.columns``.
 
-        >>> import streamlit as st
-        >>>
-        >>> col1, col2, col3 = st.columns(3)
-        >>> col1.metric("Temperature", "70 °F", "1.2 °F")
-        >>> col2.metric("Wind", "9 mph", "-8%")
-        >>> col3.metric("Humidity", "86%", "4%")
+                >>> import streamlit as st
+                >>>
+                >>> col1, col2, col3 = st.columns(3)
+                >>> col1.metric("Temperature", "70 °F", "1.2 °F")
+                >>> col2.metric("Wind", "9 mph", "-8%")
+                >>> col3.metric("Humidity", "86%", "4%")
 
-        .. output::
-            https://doc-metric-example2.streamlit.app/
-            height: 210px
+                .. output::
+                    https://doc-metric-example2.streamlit.app/
+                    height: 210px
 
-        **Example 3: Modify the delta indicator**
+                **Example 3: Modify the delta indicator**
 
-        The delta indicator color can also be inverted or turned off.
+                The delta indicator color can also be inverted or turned off.
 
-        >>> import streamlit as st
-        >>>
-        >>> st.metric(label="Gas price", value=4, delta=-0.5, delta_color="inverse")
-        >>>
-        >>> st.metric(
-        ...     label="Active developers",
-        ...     value=123,
-        ...     delta=123,
-        ...     delta_color="off",
-        ... )
+                >>> import streamlit as st
+                >>>
+                >>> st.metric(
+                ...     label="Gas price", value=4, delta=-0.5, delta_color="inverse"
+                ... )
+                >>>
+                >>> st.metric(
+                ...     label="Active developers",
+                ...     value=123,
+                ...     delta=123,
+                ...     delta_color="off",
+                ... )
 
-        .. output::
-            https://doc-metric-example3.streamlit.app/
-            height: 320px
+                .. output::
+                    https://doc-metric-example3.streamlit.app/
+                    height: 320px
 
-        **Example 4: Create a grid of metric cards**
+                **Example 4: Create a grid of metric cards**
 
-        Add borders to your metrics to create a dashboard look.
+                Add borders to your metrics to create a dashboard look.
 
-        >>> import streamlit as st
-        >>>
-        >>> a, b = st.columns(2)
-        >>> c, d = st.columns(2)
-        >>>
-        >>> a.metric("Temperature", "30°F", "-9°F", border=True)
-        >>> b.metric("Wind", "4 mph", "2 mph", border=True)
-        >>>
-        >>> c.metric("Humidity", "77%", "5%", border=True)
-        >>> d.metric("Pressure", "30.34 inHg", "-2 inHg", border=True)
+                >>> import streamlit as st
+                >>>
+                >>> a, b = st.columns(2)
+                >>> c, d = st.columns(2)
+                >>>
+                >>> a.metric("Temperature", "30°F", "-9°F", border=True)
+                >>> b.metric("Wind", "4 mph", "2 mph", border=True)
+                >>>
+                >>> c.metric("Humidity", "77%", "5%", border=True)
+                >>> d.metric("Pressure", "30.34 inHg", "-2 inHg", border=True)
 
-        .. output::
-            https://doc-metric-example4.streamlit.app/
-            height: 350px
+                .. output::
+                    https://doc-metric-example4.streamlit.app/
+                    height: 350px
 
-        **Example 5: Show sparklines**
+                **Example 5: Show sparklines**
 
-        To show trends over time, add sparklines.
+                To show trends over time, add sparklines.
 
-        >>> import streamlit as st
-        >>> from numpy.random import default_rng as rng
-        >>>
-        >>> changes = list(rng(4).standard_normal(20))
-        >>> data = [sum(changes[:i]) for i in range(20)]
-        >>> delta = round(data[-1], 2)
-        >>>
-        >>> row = st.container(horizontal=True)
-        >>> with row:
-        >>>     st.metric(
-        ...         "Line", 10, delta, chart_data=data, chart_type="line", border=True
-        ...     )
-        >>>     st.metric(
-        ...         "Area", 10, delta, chart_data=data, chart_type="area", border=True
-        ...     )
-        >>>     st.metric(
-        ...         "Bar", 10, delta, chart_data=data, chart_type="bar", border=True
-        ...     )
+                >>> import streamlit as st
+                >>> from numpy.random import default_rng as rng
+                >>>
+                >>> changes = list(rng(4).standard_normal(20))
+                >>> data = [sum(changes[:i]) for i in range(20)]
+                >>> delta = round(data[-1], 2)
+                >>>
+                >>> row = st.container(horizontal=True)
+                >>> with row:
+                >>>     st.metric(
+                ...         "Line", 10, delta, chart_data=data, chart_type="line", border=True
+                ...     )
+                >>>     st.metric(
+                ...         "Area", 10, delta, chart_data=data, chart_type="area", border=True
+                ...     )
+                >>>     st.metric(
+                ...         "Bar", 10, delta, chart_data=data, chart_type="bar", border=True
+                ...     )
 
-        .. output::
-            https://doc-metric-example5.streamlit.app/
-            height: 300px
+                .. output::
+                    https://doc-metric-example5.streamlit.app/
+                    height: 300px
 
         """
         maybe_raise_label_warnings(label, label_visibility)
@@ -324,6 +402,9 @@ class MetricMixin:
 
         metric_proto.chart_type = _parse_chart_type(chart_type)
 
+        if format is not None:
+            metric_proto.format = format
+
         validate_height(height, allow_content=True)
         validate_width(width, allow_content=True)
         layout_config = LayoutConfig(width=width, height=height)
@@ -381,10 +462,11 @@ def _determine_delta_color_and_direction(
     delta_color: DeltaColor,
     delta: Delta,
 ) -> MetricColorAndDirection:
-    if delta_color not in {"normal", "inverse", "off"}:
+    if delta_color not in _VALID_DELTA_COLORS:
         raise StreamlitAPIException(
             f"'{delta_color}' is not an accepted value. delta_color only accepts: "
-            "'normal', 'inverse', or 'off'"
+            "'normal', 'inverse', 'off', or a color name ('red', 'orange', 'yellow', "
+            "'green', 'blue', 'violet', 'gray'/'grey', 'primary')"
         )
 
     if delta is None or delta == "":
@@ -393,22 +475,36 @@ def _determine_delta_color_and_direction(
             direction=MetricProto.MetricDirection.NONE,
         )
 
-    if _is_negative_delta(delta):
-        if delta_color == "normal":
-            cd_color = MetricProto.MetricColor.RED
-        elif delta_color == "inverse":
-            cd_color = MetricProto.MetricColor.GREEN
-        else:
-            cd_color = MetricProto.MetricColor.GRAY
-        cd_direction = MetricProto.MetricDirection.DOWN
-    else:
-        if delta_color == "normal":
-            cd_color = MetricProto.MetricColor.GREEN
-        elif delta_color == "inverse":
-            cd_color = MetricProto.MetricColor.RED
-        else:
-            cd_color = MetricProto.MetricColor.GRAY
-        cd_direction = MetricProto.MetricDirection.UP
+    # Determine direction based on delta sign
+    cd_direction = (
+        MetricProto.MetricDirection.DOWN
+        if _is_negative_delta(delta)
+        else MetricProto.MetricDirection.UP
+    )
+
+    # Handle explicit color names
+    if delta_color in _DELTA_COLOR_TO_PROTO:
+        return MetricColorAndDirection(
+            color=_DELTA_COLOR_TO_PROTO[delta_color],
+            direction=cd_direction,
+        )
+
+    # Handle "normal", "inverse", "off" modes
+    is_negative = cd_direction == MetricProto.MetricDirection.DOWN
+    if delta_color == "normal":
+        cd_color = (
+            MetricProto.MetricColor.RED
+            if is_negative
+            else MetricProto.MetricColor.GREEN
+        )
+    elif delta_color == "inverse":
+        cd_color = (
+            MetricProto.MetricColor.GREEN
+            if is_negative
+            else MetricProto.MetricColor.RED
+        )
+    else:  # "off"
+        cd_color = MetricProto.MetricColor.GRAY
 
     return MetricColorAndDirection(
         color=cd_color,
