@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,6 +38,7 @@ from streamlit.elements.lib.options_selector_utils import (
     create_mappings,
     get_default_indices,
     maybe_coerce_enum_sequence,
+    validate_and_sync_multiselect_value_with_options,
 )
 from streamlit.elements.lib.policies import (
     check_widget_policies,
@@ -493,15 +494,9 @@ class MultiSelectMixin:
         element_id = compute_and_register_element_id(
             widget_name,
             user_key=key,
-            # Treat the provided key as the main identity. Only include
-            # changes to the options, accept_new_options, and max_selections
-            # in the identity computation as those can invalidate the
-            # current selection.
             key_as_main_identity={
-                "options",
                 "max_selections",
                 "accept_new_options",
-                "format_func",
             },
             dg=self.dg,
             label=label,
@@ -554,8 +549,23 @@ class MultiSelectMixin:
             widget_state, options, indexable_options
         )
 
-        if widget_state.value_changed:
-            proto.raw_values[:] = serde.serialize(widget_state.value)
+        if accept_new_options:
+            # accept_new_options is True, so we keep the user-entered values.
+            current_values = widget_state.value
+            value_needs_reset = False
+        else:
+            # Validate the current values against the new options.
+            # If values are no longer valid (not in options), filter them out.
+            # This handles the case where options change dynamically and the
+            # previously selected values are no longer available.
+            current_values, value_needs_reset = (
+                validate_and_sync_multiselect_value_with_options(
+                    widget_state.value, indexable_options, key
+                )
+            )
+
+        if value_needs_reset or widget_state.value_changed:
+            proto.raw_values[:] = serde.serialize(current_values)
             proto.set_value = True
 
         validate_width(width)
@@ -566,7 +576,7 @@ class MultiSelectMixin:
 
         self.dg._enqueue(widget_name, proto, layout_config=layout_config)
 
-        return widget_state.value
+        return current_values
 
     @property
     def dg(self) -> DeltaGenerator:
